#ifndef CHESSLIB_H
#define CHESSLIB_H

/* For basic C functions. */
#include <stdlib.h>

/* For time. */
#include <time.h>

/* Give each chess piece a value.  Keeps the code cleaner. */
#define B_ROOK 1
#define B_KNIGHT 2
#define B_BISHOP 3
#define B_QUEEN 4
#define B_KING 5
#define B_PAWN 6
#define W_ROOK 7
#define W_KNIGHT 8
#define W_BISHOP 9
#define W_QUEEN 10
#define W_KING 11
#define W_PAWN 12

/* These two macros are useful for determining whether certain
 * moves are legal.
 */
#define max(A,B) ((A>B) ? A : B)
#define min(A,B) ((A<B) ? A : B)

/* Here, we define a rule for determining how many plies the AI
 * should search.
 */
#define PLIES (bAIDiff+2)

/* A move can be stored in a struct to make AI and networked
 * games easier.
 * 	y2, x2 : start location
 * 	y, x : end location
 * 	type : type of chess piece
 * 	value : value of the move (for AI)
 * 	next : next move in the list (for AI)
 */
struct move
{
	int y2;
	int x2;
	int y;
	int x;
	int type;
	int value;
	struct move* next;
};

/* Since moves can be stored in a list, we need methods to add and
 * remove them from the list.  Push adds, pop removes.  Lists work in a
 * last-in-first-out (LIFO) fashion.
 */
void push(struct move** refHead, int value, int y2, int x2, int y, int x, int type);
void pop(struct move** refHead);

/* The following 12 functions estalbish the rules for moving the 12
 * different chess pieces.  Each function is called move[color][type]
 * where color is B for black or W for white and type is the type of
 * chess piece to move.  These functions simply determine whether a move
 * is legal (or, in special circustances, whether extra actions need to
 * be performed for said legal move (ex. castling)) and do not actually
 * move the chess pieces.
 *      y2, x2: the piece's starting location (GL coords)
 *      y, x : the piece's desired new location (GL coords)
 *      turn2 : who's turn is it (theoretically)?
 */
int moveBRook(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveBKnight(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveBBishop(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveBQueen(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveBKing(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*hasMoved)[8], int (*enPassant)[8]);
int moveBPawn(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*enPassant)[8]);
int moveWRook(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveWKnight(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveWBishop(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveWQueen(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8]);
int moveWKing(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*hasMoved)[8], int (*enPassant)[8]);
int moveWPawn(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*enPassant)[8]);

/* This method determines whether or not a king is in check.
 *      ky, kx : the king's position in chessboard
 *      correctTurn : who's turn is it (theoretically)?
 */
int inCheck(int ky, int kx, int correctTurn, int (*chessboard)[8], int (*hasMoved)[8], int (*enPassant)[8]);

/* This pair of functions makes and unmakes a move.
 * 	y2, x2 : starting location
 * 	y, x : destination
 * 	turn2 : turn (theoretically)
 * 	flag : the type of move (normal, castle, etc.)
 * 	pieceHolder : type of captured piece, if any
 */
int makeMove(int y2, int x2, int y, int x, int turn2, int flag, int* bCastled, int* wCastled, int (*hasMoved)[2][8], int (*chessboard)[8][8], int (*enPassant)[2][8]);
void unmakeMove(int y2, int x2, int y, int x, int turn2, int flag, int pieceHolder, int* bCastled, int* wCastled, int (*hasMoved)[2][8], int (*chessboard)[8][8], int (*enPassant)[2][8]);

/* Determines whether a given move is valid.  Does not actually
 * move any chess pieces.
 */
int isValidMove(int type, int y2, int x2, int y, int x, int turn2, int bCastled, int wCastled, int (*chessboard)[8][8], int (*hasMoved)[2][8], int (*enPassant)[2][8]);

/* Gets a list of valid moves for a player.
 *      turn2 : who's turn is it (theoretically)?
 */
struct move* getLegalMoves(int turn2, int bCastled, int wCastled, int (*chessboard)[8][8], int (*hasMoved)[2][8], int (*enPassant)[2][8]);

/* Does things that need to be done after making a valid move. */
void doMove(int captured, int* illegalMove, int* turn, int* wturns, int* bturns, int* check, int* mate, int* wtime, int* btime, int* sttime, int* wCastled, int* bCastled, int (*wCap)[15], int (*bCap)[15], int (*hasMoved)[2][8], int (*chessboard)[8][8], int (*enPassant)[2][8]);

/* Evaluates a chessboard.  This helps the AI determine which moves
 * are best.
 */
int evaluate(int turn2);

/* Cycles through all possible moves and chooses the best one (AI).
 * This function calls itself recursively, allowing the AI to look
 * into the future a few moves.
 */
int negaMax(int depth, int alpha, int beta, int turn2);

#endif /* CHESSLIB_H */
