#ifndef CLIENT
#define CLIENT

/* For OpenGL. */
#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>

/* For usleep() and alarm(). */
#include <unistd.h>

/* For signal(). */
#include <signal.h>

/* For basic C functions and I/O. */
#include <stdlib.h>
#include <stdio.h>

/* For strlen(). */
#include <string.h>

/* For functions with a variable number of parameters. */
#include <stdarg.h>

/* For time(). */
#include <time.h>

/* For abs(). */
#include <math.h>

/* For debugging. */
#include <assert.h>

/* Networkin' */
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* Allows for easy changing of input keys.
 *      ESCAPE: exits the game
 *      UP: moves the cursor up
 *      DOWN: moves the cursor down
 *      LEFT: moves the cursor left
 *      RIGHT: moves the cursor right
 *      SPACE_BAR: selects the square to move from or to
 *      EP_DEBUG: prints enPassant to stderr (for debug)
 *      HM_DEBUG: prints hasMoved to stderr (for debug)
 *      B_DEBUG: prints chessboard to stderr (for debug)
 *      CAP_DEBUG: prints bCap and wCap to stderr (for debug)
 *	GET_MOVES: prints a list of legal moves (for debug)
 */
#define ESCAPE 27
#define UP 'w'
#define DOWN 's'
#define LEFT 'a'
#define RIGHT 'd'
#define SPACE_BAR 32
#ifdef DKEYS
	#define EP_DEBUG 'i'
	#define HM_DEBUG 'o'
	#define B_DEBUG 'p'
	#define CAP_DEBUG 'u'
	#define GET_MOVES 'k'
#endif /* DKEYS */

/* Main return codes */
#define SUCCESS 0
#define BADARG 1
#define HOSTERR 10
#define REMOTEHOSTERR 12
#define SOCKERR 13
#define RECVERR 14
#define AIERR 15
#define NAMEERR 16

/* Maximum command length - this should be the same server-side too */
#define MAXCMD 4096

void keyPressed(unsigned char key, int x, int y);

/* The buildFont() function sets up font lists.  Any number of
 * fonts can be supported, though they must be declared globally
 * and handled in this function.  Be sure to call XFreeFont()
 * after setting up each font.
 */
void buildFont(void);

/* For deleting the font lists.  Make sure to call this function
 * before exiting the program, and to add a call to glDeleteLists()
 * for each font here!
 */
void killFont(void);

/* This is a custom print routine for fonts.  This function accepts
 * text formatting exactly like the sprintf() function found in stdio.
 *      text : the text to print
 *      font : the font to use, starting with 0 and progressing in the
 *	      order that fonts were created
 *      ... : any other arguments passed to glPrint(), allowing it to
 *	      behave like sprintf().
 */
void glPrint(const char* text, int font, ...);


void playGame(void);

/* For resizing the window.  This keeps aspect ratios and perspectives
 * looking nice when the user decides to change the window's size.
 *      width : the new window width
 *      height : the new window height
 */
void resizeGLScene(int width, int height);

/* initGL() initializes our OpenGL window. */
int initGL(int width, int height);

/* The main drawing function.  This function is called continuously while
 * the program is running.  Each call to this function is effectively one
 * frame on the screen.  So, the number of times we can loop through this
 * per second gives us our frames per second (FPS).  There is no reason
 * to ever call drawGLScene() manually.
 */
void drawGLScene(void);

void parseMove(char* buffer);
char* unparseMove(void);

void initChessboard(void);

/**
 * Prints the usage information for the client program.
 */
void usage(void);

/**
 * Does everything aside form the parameter processing
 *
 * @param server The server to connect to
 * @param port The port on the server to connect to
 * @param pass The password to supply to the server
 * @param cmd The cmd to run, or NULL for semi-interactive/broken mode
 * @return One of the return codes above
 */
int socketStuff(const char* server, const int port, const char* name);

/* For handling alarm signals (update UI signals). */
void sigAlarmHandler(int signum);

#endif /* CLIENT */
